# Future
from __future__ import division, print_function, unicode_literals

PER_PAGE_MAX = 100
BULK_LIMIT = 25
BASE_URI = "https://api.beta.documentcloud.org/api/"
AUTH_URI = "https://accounts.muckrock.com/api/"
TIMEOUT = 10
RATE_LIMIT = 10
RATE_PERIOD = 1
